<div class="peek-nav-section">
    <!--
        The following 'div' groups button to the left of the Nav Bar.
        Can contain one to many buttons
    -->
    <div class="btn-group pull-left" role="group"></div>

    <!--
        The following 'div' groups button to the right of the Nav Bar.
        Can contain one to many buttons
    -->
    <div class="btn-group pull-right" role="group">
        <button
            (click)="doLogout()"
            [disabled]="isAuthenticating"
            class="nav-section-btn"
            role="group"
        >
            <i nz-icon nzTheme="outline" nzType="logout"></i>
            Logout
        </button>
    </div>
</div>

<div class="peek-nav-bar-padding">
    <!-- This div provides padding for the screen under the Navigation Section -->
</div>

<div class="plugin-user">
    <div *ngFor="let error of errors" class="bg-danger">{{error}}</div>

    <div *ngFor="let warning of warnings" class="bg-warning">{{warning}}</div>

    <div *ngIf="warnings != null && warnings.length != 0" class="bg-info">
        Click 'Logout' a second time to ignore these warnings.
    </div>

    <div class="user-message">
        <i nz-icon nzTheme="outline" nzType="login"></i>
        Logged in as {{loggedInUserText()}}
    </div>
</div>
